Package: opencis
Type: Package
Title: Import Data from Spanish Sociological Research Center (CIS)
Version: 0.1.1
Authors@R: c(
    person("Héctor", "Meleiro", email = "hmeleiros@gmail.com", role = c("aut", "cre"))
  )
Description: Search and import data directly to R from the Spanish Sociological
    Research Center (CIS) <https://www.cis.es/inicio>. The CIS is a public
    institution that conducts electoral and sociological research studies on the
    Spanish society. The CIS has a large database of surveys that can be
    accessed through its website. The package includes functions to search for
    surveys, survey questions and timeseries, and import the data directly to R.
License: GPL (>= 3)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: httr (>= 1.4.7), tibble (>= 3.0.0), purrr (>= 1.0.0), haven
        (>= 2.5.3), magrittr (>= 2.0.0), rvest (>= 1.0.0), stringr (>=
        1.0.0), memoise (>= 2.0.0)
URL: https://opencis.spainelectoralproject.com,
        https://CRAN.R-project.org/package=opencis,
        https://github.com/hmeleiro/opencis
BugReports: https://github.com/hmeleiro/opencis/issues
Suggests: knitr, testthat (>= 3.0.0), rmarkdown
VignetteBuilder: knitr
Config/Needs/website: hmeleiro/spainelectoraltheme
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-12 09:12:23 UTC; hmele
Author: Héctor Meleiro [aut, cre]
Maintainer: Héctor Meleiro <hmeleiros@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 18:40:25 UTC
Built: R 4.6.0; ; 2026-05-12 19:54:28 UTC; unix

# Generated by roxygen2: do not edit by hand

export(browse_pdf)
export(clear_cache)
export(download_study)
export(get_data_dictionary)
export(get_metadata)
export(read_cis)
export(search_all_cis)
export(search_cis)
importFrom(haven,read_sav)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(httr,modify_url)
importFrom(httr,status_code)
importFrom(httr,stop_for_status)
importFrom(httr,write_disk)
importFrom(magrittr,"%>%")
importFrom(memoise,forget)
importFrom(memoise,memoise)
importFrom(purrr,list_rbind)
importFrom(purrr,map)
importFrom(rvest,html_attr)
importFrom(rvest,html_element)
importFrom(rvest,html_elements)
importFrom(rvest,html_text)
importFrom(stringr,str_extract_all)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,browseURL)
importFrom(utils,unzip)

# opencis <img src="man/figures/logo.png" align="right" height="139" alt="" />

<!-- badges: start -->
[![CRAN version](https://www.r-pkg.org/badges/version/opencis)](https://CRAN.R-project.org/package=opencis)
[![R-CMD-check](https://github.com/hmeleiro/opencis/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/hmeleiro/opencis/actions/workflows/R-CMD-check.yaml)
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

Search and import data directly to R from the Spanish Sociological Research Center (CIS). The CIS is a public institution that conducts electoral and sociological research studies on the Spanish society. The CIS has a large database of surveys that can be accessed through its website. The package includes functions to search for surveys, survey questions and timeseries, and import the data directly to R.

## Installation

You can install the released version of `opencis` from CRAN:

``` r
install.packages("opencis")
```

You can install the development version from GitHub using the `remotes` package:

``` r
remotes::install_github("hmeleiro/opencis")
```

## Usage

The package includes two functions.

The first function is used to search for surveys, survey questions and data series

``` r
# Search for survey studies
library(opencis)

# Search for survey studies
search_cis(q = "preelectoral", from = "2020-01-01", to = "2023-11-17")

#> # A tibble: 27 × 4
#>    study   date       title                                                              url  
#>    <chr>   <date>     <chr>                                                              <chr>
#>  1 3411    2023-06-08 Preelectoral elecciones generales 2023                             http…
#>  2 3402    2023-04-10 Preelectoral elecciones municipales y autonómicas 2023             http…
#>  3 3306    2021-01-02 Preelectoral de Cataluña. Elecciones autonómicas 2021              http…
#>  4 3317    2021-03-19 Preelectoral elecciones autonómicas 2021. Comunidad de Madrid      http…
#>  5 3287    2020-06-10 Preelectoral de Galicia. Elecciones autonómicas julio 2020         http…
#>  6 3276    2020-02-17 Preelectoral de Galicia. Elecciones autonómicas abril 2020         http…
#>  7 3286    2020-06-10 Preelectoral del País Vasco. Elecciones autonómicas julio 2020     http…
#>  8 3275    2020-02-17 Preelectoral del País Vasco. Elecciones autonómicas abril 2020     http…
#>  9 3365    2022-05-17 Preelectoral elecciones autonómicas 2022. Comunidad autónoma de A… http…
#> 10 3348    2022-01-07 Preelectoral elecciones autonómicas 2022. Comunidad autónoma de C… http…
#> # ℹ 17 more rows
#> # ℹ Use `print(n = ...)` to see more rows


# Search for survey questions
search_cis(q = "feminismo", catalogo = "pregunta")

#> # A tibble: 8 × 4
#>   question             date       title                                                  url  
#>   <chr>                <date>     <chr>                                                  <chr>
#> 1 Pregunta 3428/0/0016 2023-11-06 Escala de autodefinición feminista (0-10)              http…
#> 2 Pregunta 2212/0/0008 1996-04-13 Sentimientos hacia diferentes ideologías, situaciones… http…
#> 3 Pregunta 2828/0/0010 2010-01-09 Rasgos ideológicos que definen a la derecha y a la iz… http…
#> 4 Pregunta 1993/0/0009 1992-02-27 Sentimientos hacia diferentes ideologías, situaciones… http…
#> 5 Pregunta 2401/0/0025 2000-12-09 Rasgos ideológicos que definen a la derecha y a la iz… http…
#> 6 Pregunta 3428/0/0010 2023-11-06 Aprobación de diferentes conductas pro-igualdad de gé… http…
#> 7 Pregunta 2016/0/0026 1992-07-09 Rasgos que definen a la derecha y a la izquierda polí… http…
#> 8 Pregunta 2154/0/0036 1995-04-03 Rasgos que definen a la derecha y a la izquierda polí… http…

# Search for data series
search_cis(q = "situación económica", catalogo = "serie")

#> # A tibble: 100 × 5
#>    serie               from       to         title                                                               url    
#>    <chr>               <date>     <date>     <chr>                                                               <chr>  
#>  1 Serie K.1.03.03.012 2011-11-21 2025-09-12 Situación económica familiar (ICC)                                  https:…
#>  2 Serie K.1.02.01.014 1992-11-13 2019-10-01 Valoración de la situación económica actual de España (Extremadura) https:…
#>  3 Serie K.1.02.01.008 1992-11-14 2019-06-17 Valoración de la situación económica actual de España (Cantabria)   https:…
#>  4 Serie K.1.02.01.003 1986-05-01 2019-10-01 Valoración de la situación económica actual de España (Andalucía)   https:…
#>  5 Serie K.1.02.01.004 1998-05-01 2019-10-01 Valoración de la situación económica actual de España (Aragón)      https:…
#>  6 Serie K.1.02.01.007 1998-05-01 2019-10-01 Valoración de la situación económica actual de España (Canarias)    https:…
#>  7 Serie K.1.02.01.019 1998-05-01 2015-05-29 Valoración de la situación económica actual de España (La Rioja)    https:…
#>  8 Serie K.1.02.01.011 1992-11-07 2020-03-01 Valoración de la situación económica actual de España (Cataluña)    https:…
#>  9 Serie K.1.03.01.012 2010-01-23 2017-09-21 Valoración de la situación económica personal actual (Andalucía)    https:…
#> 10 Serie K.1.03.01.020 1992-04-24 2025-10-01 Valoración de la situación económica personal actual (Nacional)     https:…
#> # ℹ 90 more rows
#> # ℹ Use `print(n = ...)` to see more rows
```

The second function is used to import the data directly into R.

``` r
# Read a survey study
df <- read_cis(3411)
print(df)

#> # A tibble: 29,201 × 216
#>    ESTUDIO     REGISTRO  CUES CCAA    PROV    MUN     CAPITAL TAMUNI  ENTREV  TIPO_TEL SEXO   
#>    <dbl+lbl>      <dbl> <dbl> <dbl+l> <dbl+l> <dbl+l> <dbl+l> <dbl+l> <dbl+l> <dbl+lb> <dbl+l>
#>  1 3411 [3411]   492777 29043 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 1 [Men… 0 [Ano… 2 [Móvi… 1 [Hom…
#>  2 3411 [3411]    70655  5848 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 1 [Men… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  3 3411 [3411]    46423  3893 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  4 3411 [3411]   103464  8711 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  5 3411 [3411]   112810  9517 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  6 3411 [3411]   126567 10736 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 1 [Hom…
#>  7 3411 [3411]   137692 11573 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  8 3411 [3411]   182178 14839 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#>  9 3411 [3411]   216764 16793 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 1 [Fijo] 2 [Muj…
#> 10 3411 [3411]   237655 17918 1 [And… 4 [Alm… 0 [Mun… 3 [Otr… 3 [10.… 0 [Ano… 2 [Móvi… 2 [Muj…
#> # ℹ 29,191 more rows
#> # ℹ 205 more variables: EDAD <dbl+lbl>, P0A <dbl+lbl>, ECOPER <dbl+lbl>, ECOESP <dbl+lbl>,
#> #   MEDIO_1 <dbl+lbl>, MEDIO_2 <dbl+lbl>, LEEPRENSA <dbl+lbl>, VETELE <dbl+lbl>,
#> #   OYERADIO <dbl+lbl>, PRENSA <dbl+lbl>, P3AR <dbl+lbl>, TELEVISION <dbl+lbl>,
#> #   P3BR <dbl+lbl>, RADIO <dbl+lbl>, P3CR <dbl+lbl>, GESTIONGOB <dbl+lbl>,
#> #   GESTIONOPO <dbl+lbl>, PROBVOTO <dbl+lbl>, VOTOCORREO <dbl+lbl>, PROBPARTIDOS_1 <dbl+lbl>,
#> #   PROBPARTIDOS_2 <dbl+lbl>, PROBPARTIDOS_3 <dbl+lbl>, PROBPARTIDOS_4 <dbl+lbl>, …
#> # ℹ Use `print(n = ...)` to see more rows, and `colnames()` to see all variable names

```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)


## ----eval=FALSE---------------------------------------------------------------
# devtools::install_github("hmeleiro/opencis")

## ----eval = FALSE-------------------------------------------------------------
# library(opencis)
# 
# # Search for survey studies
# search_cis(q = "preelectoral", from = "2020-01-01", to = "2023-11-17")
# 
# # Search for survey questions
# search_cis(q = "feminismo", catalogo = "pregunta")
# 
# # Search for data series
# search_cis(q = "situación económica", catalogo = "serie")

## ----eval = FALSE-------------------------------------------------------------
# # Retrieve all postelectoral studies (all pages)
# all_studies <- search_all_cis(q = "postelectoral")
# print(nrow(all_studies))
# 
# # Filter by date range across all pages
# studies <- search_all_cis(
#   q    = "ideologia",
#   from = "2010-01-01",
#   to   = "2020-12-31"
# )

## ----eval = FALSE-------------------------------------------------------------
# df <- read_cis(3411)
# print(df)

## ----eval = FALSE-------------------------------------------------------------
# df   <- read_cis(3328)
# dict <- get_data_dictionary(df)
# print(dict)
# 
# # Find variables whose label contains a keyword
# dict[grepl("sexo", dict$label, ignore.case = TRUE), ]
# 
# # Inspect value labels for a specific variable
# dict$value_labels[[which(dict$variable == "SEXO")]]

## ----eval = FALSE-------------------------------------------------------------
# meta <- get_metadata(3328)
# print(meta)
# 

## ----eval = FALSE-------------------------------------------------------------
# # Save to the current working directory
# path <- download_study(3328)
# cat("Saved to:", path, "\n")
# 
# # Save to a specific folder
# path <- download_study(3328, destdir = "data/raw")
# cat("Saved to:", path, "\n")

## ----eval = FALSE-------------------------------------------------------------
# # Open the questionnaire for study 3328
# browse_pdf(3328)
# 
# # Open the technical sheet
# browse_pdf(3328, wanted_file = "ft")

#' Import a CIS study
#'
#' Download and import the data of a CIS study.
#'
#' @param study_code A string with the study code.
#'
#' @importFrom haven read_sav
#' @importFrom utils unzip
#'
#' @return A data.frame with the study data.
#'
#' @export
#'
#' @example R/examples/read_cis.R
read_cis <- function(study_code) {
  url <- get_study_url(study_code)
  zip_path <- download_file(url)
  data <- read_sav_from_zip(zip_path)

  return(data)
}

read_cis <- memoise::memoise(read_cis)


#' Download and read a CIS study from a given URL
#'
#' @param url A string with the URL of the CIS study page.
#' @param destfile A string with the path where the ZIP file will be saved.
#'   Defaults to a temporary file.
#'
#' @keywords internal
download_file <- function(url, destfile = tempfile(fileext = ".zip")) {
  resp_html <- cis_get(url, required = TRUE, context = "downloading study page")
  html <- content(resp_html, as = "text", encoding = "UTF-8")

  zip_url <- find_url(html)
  if (length(zip_url) == 0) {
    stop("No ZIP URL found on the study page. The page structure may have changed.")
  }

  resp_zip <- cis_get(
    zip_url,
    write_disk(destfile, overwrite = TRUE),
    required = TRUE,
    context = "downloading study ZIP"
  )

  if (is.null(resp_zip)) {
    stop("Could not download ZIP file.", call. = FALSE)
  }

  return(destfile)
}

#' Find CIS study data URL in HTML content
#'
#' Searches for the URL of the CIS study data ZIP file within the provided HTML content.
#'
#' @param html A character string containing the HTML content to search.
#' @param ids An optional vector of two strings representing the numeric subroutes of the URL
#'           (e.g., c("3411", "3411") for "https://www.cis.es/documents/3411/3411/MD3411.zip").
#'           If NULL, the function searches for any valid CIS study data URL.
#' @param allow_uuid A boolean indicating whether to allow UUIDs in the URL.
#'                  Defaults to FALSE.
#'
#' @return A character vector of unique URLs found in the HTML content.
#'
#' @keywords internal
find_url <- function(html,
                     ids = NULL, # subrutas numéricas
                     allow_uuid = FALSE) {
  stopifnot(is.character(html), length(html) == 1)

  base_pat <- if (is.null(ids)) {
    "https://www\\.cis\\.es/documents/\\d+/\\d+/"
  } else {
    sprintf("https://www\\.cis\\.es/documents/%s/%s/", ids[1], ids[2])
  }

  md_part <- "MD\\d+\\.zip"
  uuid_part <- if (allow_uuid) "(?:/[A-Za-z0-9-]+)?" else ""

  pat <- paste0(base_pat, md_part, uuid_part)

  out <- unlist(str_extract_all(html, pat))
  unique(out[!is.na(out) & nzchar(out)])
}


#' Read a SAV file from a ZIP archive
#'
#' Extracts and reads the SPSS (.sav) data file contained in a ZIP archive.
#'
#' @param zip_path A string with the path to the ZIP file.
#'
#' @return A data.frame with the data read from the .sav file.
#'
#' @keywords internal
read_sav_from_zip <- function(zip_path) {
  sav_file <- list_file_paths(zip_path, type = "sav")

  if (nrow(sav_file) == 1) {
    out_dir <- tempdir()
    sav_path <- unzip(zip_path, files = sav_file$Name, exdir = out_dir)

    data <- read_sav(sav_path, user_na = TRUE)
    return(data)
  } else if (nrow(sav_file) > 1) {
    stop("Found more than one sav file in zip file.")
  } else {
    stop("No sav file found.")
  }
}


#' List file paths inside a ZIP archive
#'
#' Returns a data.frame with the files contained in a ZIP archive,
#' optionally filtered by file extension.
#'
#' @param zip_file A string with the path to the ZIP file.
#' @param type A string with the file extension to filter by (e.g. \code{"sav"}, \code{"pdf"}).
#'   If \code{NULL} (default), all files are returned.
#'
#' @return A data.frame with the files in the ZIP archive.
#'
#' @keywords internal
list_file_paths <- function(zip_file, type = NULL) {
  files_in_zip <- unzip(zip_file, list = TRUE)

  if (is.null(type)) {
    return(files_in_zip)
  }

  files_type <- files_in_zip[grepl(sprintf("\\.%s$", type), files_in_zip$Name), ]
  return(files_type)
}


#' Open the questionnaire PDF of a CIS study
#'
#' Opens a PDF document from a CIS study in the default browser.
#'
#' CIS study ZIP files typically contain two PDF documents:
#' \itemize{
#'   \item The \strong{questionnaire} (cuestionario): use \code{wanted_file = "cues"}.
#'   \item The \strong{technical sheet} (ficha técnica): use \code{wanted_file = "ft"}.
#' }
#'
#' @param study_code A string with the study code.
#' @param wanted_file A keyword used to match the PDF filename inside the ZIP.
#'   Use \code{"cues"} (default) for the questionnaire or \code{"ft"} for the
#'   technical sheet.
#'
#' @return Called for its side effect of opening the PDF in the browser.
#'   Returns \code{NULL} invisibly.
#'
#' @export
#'
#' @example R/examples/browse_pdf.R
browse_pdf <- function(study_code, wanted_file = "cues") {
  url <- get_study_url(study_code)
  zip_path <- download_file(url)

  pdf_files <- list_file_paths(zip_path, "pdf")

  pdf_file <- pdf_files[grepl(wanted_file, pdf_files$Name, ignore.case = TRUE), ]

  if (nrow(pdf_file) == 1) {
    out_dir <- tempdir()
    pdf_path <- unzip(zip_path, files = pdf_file$Name, exdir = out_dir)
    browseURL(pdf_path)
  } else if (nrow(pdf_file) > 1) {
    stop(sprintf("Found more than one pdf file in zip file with keyword '%s'.", wanted_file))
  } else {
    stop(sprintf("No pdf file found in zip file with keyword '%s'.", wanted_file))
  }

  invisible(NULL)
}


#' Download a CIS study ZIP file to disk
#'
#' Downloads the data ZIP file for a CIS study to a specified directory,
#' instead of a temporary folder. Useful for projects that need to keep
#' the raw data files.
#'
#' @param study_code A string with the study code.
#' @param destdir A string with the directory where the ZIP file will be saved.
#'   Defaults to the current working directory.
#'
#' @return The path to the saved ZIP file, invisibly.
#'
#' @export
#'
#' @example R/examples/download_study.R
download_study <- function(study_code, destdir = ".") {
  url <- get_study_url(study_code)
  if (is.null(url)) {
    stop("Study '", study_code, "' not found.")
  }

  if (!dir.exists(destdir)) {
    stop("Directory does not exist: ", destdir)
  }

  destfile <- file.path(destdir, paste0("MD", study_code, ".zip"))
  path <- download_file(url, destfile = destfile)

  invisible(normalizePath(path))
}


#' Extract a data dictionary from a CIS study data frame
#'
#' Returns a tibble listing each variable in the data along with its
#' variable label and value labels, as loaded by \code{haven}.
#'
#' @param data A data.frame loaded from a CIS \code{.sav} file,
#'   typically the output of \code{\link{read_cis}}.
#'
#' @return A tibble with columns:
#'   \describe{
#'     \item{variable}{Variable name.}
#'     \item{label}{Variable label, or \code{NA} if none.}
#'     \item{value_labels}{A named numeric vector of value labels,
#'       or \code{NULL} for unlabelled variables (list-column).}
#'   }
#'
#' @export
#'
#' @example R/examples/get_data_dictionary.R
get_data_dictionary <- function(data) {
  if (!is.data.frame(data)) stop("'data' must be a data.frame.")

  vl <- lapply(data, function(x) attr(x, "label"))
  vv <- lapply(data, function(x) attr(x, "labels"))

  normalize_label <- function(x) {
    if (is.null(x) || length(x) == 0) {
      return(NA_character_)
    }

    x <- as.character(x)
    x <- x[!is.na(x) & nzchar(x)]

    if (length(x) == 0) {
      return(NA_character_)
    }
    x[[1]]
  }

  tibble(
    variable     = names(data),
    label        = vapply(vl, normalize_label, character(1)),
    value_labels = vv
  )
}

test_that("get_data_dictionary returns expected structure", {
    x <- structure(c(1, 2), label = "Sexo", labels = c(Hombre = 1, Mujer = 2))
    y <- c("A", "B")
    df <- data.frame(SEXO = x, LETRA = y)

    dict <- opencis::get_data_dictionary(df)

    expect_s3_class(dict, "tbl_df")
    expect_equal(nrow(dict), 2)
    expect_equal(dict$variable, c("SEXO", "LETRA"))
    expect_equal(dict$label[[1]], "Sexo")
    expect_true(is.na(dict$label[[2]]))
})

test_that("find_url extracts matching CIS ZIP links", {
    html <- paste(
        '<a href="https://www.cis.es/documents/3411/3411/MD3411.zip">zip</a>',
        '<a href="https://www.cis.es/documents/3328/3328/MD3328.zip">zip2</a>'
    )

    out <- opencis:::find_url(html)
    expect_length(out, 2)
    expect_true(all(grepl("^https://www.cis.es/documents/", out)))

    out_one <- opencis:::find_url(html, ids = c("3411", "3411"))
    expect_identical(out_one, "https://www.cis.es/documents/3411/3411/MD3411.zip")
})

# Create a small labelled data frame
df <- data.frame(
  SEXO = haven::labelled(c(1, 2, 1), labels = c(Hombre = 1, Mujer = 2)),
  EDAD = c(34, 51, 29)
)
attr(df$SEXO, "label") <- "Sexo"
attr(df$EDAD, "label") <- "Edad"

# Inspect its variable dictionary
dict <- get_data_dictionary(df)
print(dict)

# Find variables with a specific keyword in their label
dict[grepl("sexo", dict$label, ignore.case = TRUE), ]

# Inspect value labels for a specific variable
sex_var <- match("SEXO", dict$variable)
if (!is.na(sex_var)) {
  dict$value_labels[[sex_var]]
}

#' Clear the opencis session cache
#'
#' Clears the in-memory cache used by \code{\link{search_cis}} and
#' \code{\link{read_cis}}. Call this when you want to force fresh data
#' to be retrieved from the CIS server within the same R session.
#'
#' @return \code{NULL} invisibly.
#'
#' @export
clear_cache <- function() {
  memoise::forget(search_cis)
  memoise::forget(read_cis)
  invisible(NULL)
}

\donttest{
# If you know the study code you can just read it into R
df <- read_cis("3328")
print(df)

# If you dont know the study code, you can search for a study using search_cis() function:
studies <- search_cis(q = "gastronomia")
print(studies)

df <- read_cis(studies$study[1])
print(df)
}

#' Package imports
#'
#' These declarations ensure NAMESPACE keeps required imports.
#' @title Imports
#' @description Package import declarations
#' @name opencis-imports
#' @keywords internal
#' @importFrom httr GET status_code content write_disk stop_for_status modify_url
#' @importFrom haven read_sav
#' @importFrom utils unzip browseURL
#' @importFrom rvest html_elements html_element html_attr html_text
#' @importFrom magrittr %>%
#' @importFrom purrr map list_rbind
#' @importFrom tibble tibble as_tibble
#' @importFrom stringr str_extract_all
#' @importFrom memoise memoise forget
NULL

\donttest{
# Search by search terms
studies <- search_cis(q = "postelectoral")
print(studies)

# Narrow the search by dates
studies <- search_cis(q = "postelectoral",
                          from = "2011-01-01",
                          to = "2020-01-01")
print(studies)

# Use the catalogo parameter to search for questions ("pregunta") or data series ("serie")
studies <- search_cis(q = "ideologia",
                          from = "2011-01-01",
                          to = "2020-01-01",
                          catalogo = "serie")
print(studies)
}

cis_request_timeout <- function() {
    timeout <- getOption("opencis.timeout", 20)
    timeout <- suppressWarnings(as.numeric(timeout))
    if (is.na(timeout) || timeout <= 0) {
        return(20)
    }
    timeout
}

cis_user_agent <- function() {
    paste0("opencis/", utils::packageVersion("opencis"))
}

cis_get <- function(url, ..., required = FALSE, context = "request") {
    resp <- tryCatch(
        GET(
            url,
            httr::timeout(cis_request_timeout()),
            httr::user_agent(cis_user_agent()),
            ...
        ),
        error = function(e) {
            if (required) {
                stop(sprintf("Failed %s for '%s': %s", context, url, conditionMessage(e)), call. = FALSE)
            }
            warning(sprintf("Failed %s for '%s': %s", context, url, conditionMessage(e)), call. = FALSE)
            NULL
        }
    )

    if (is.null(resp)) {
        return(NULL)
    }

    status <- status_code(resp)
    if (status >= 400) {
        msg <- sprintf("HTTP %s while %s for '%s'.", status, context, url)
        if (required) {
            stop(msg, call. = FALSE)
        }
        warning(msg, call. = FALSE)
        return(NULL)
    }

    resp
}

#' Search for CIS studies.
#'
#' Searches for CIS studies using the CIS search engine.
#'
#' @param start Integer. The starting page for the search results. Default is 1, iterate to get more results.
#' @param q String. The search query. Default is an empty string.
#' @param from Date or NULL. The start date for filtering results. Default is NULL. The date format must be "YYYY-MM-DD".
#' @param to Date or NULL. The end date for filtering results. Default is NULL. The date format must be "YYYY-MM-DD".
#' @param sort String. The sorting order for the results ("publishDate-", "publishDate+", "relevance").
#' Default is "relevance".
#' @param catalogo String. The catalog type ("estudio", "pregunta", "serie"). Default is "estudio".
#' @param ... Additional parameters (not used).
#'
#' @return A data.frame with the search results.
#'
#' @export
#'
#' @example R/examples/search_cis.R
search_cis <- function(
  start = 1,
  q = "",
  from = NULL, # Date o NULL
  to = NULL, # Date o NULL
  sort = "relevance",
  catalogo = "estudio",
  ...
) {
  args <- as.list(environment(), all = TRUE)
  url <- do.call(cis_catalog_url_date, args)

  resp <- cis_get(url, required = FALSE, context = "searching CIS catalog")

  if (is.null(resp)) {
    return(NULL)
  }

  if (status_code(resp) == 200) {
    if (catalogo == "estudio") {
      out <- parse_study(resp)
    } else if (catalogo == "pregunta") {
      out <- parse_question(resp)
    } else if (catalogo == "serie") {
      out <- parse_serie(resp)
    } else {
      stop("Invalid catalogo value. Must be 'estudio', 'pregunta' or 'serie'.")
    }
  } else {
    return(NULL)
  }

  return(out)
}

search_cis <- memoise::memoise(search_cis)

#' Parse CIS study search results
#'
#' @param resp The HTTP response object from the CIS search.
#'
#' @return A tibble with the parsed studies information.
#'
#' @keywords internal
parse_study <- function(resp) {
  html <- content(resp, "parsed")
  a <-
    html %>%
    html_elements(".card-content") %>%
    html_element("a")


  titles <- html_attr(a, name = "title")
  urls <- html_attr(a, "href")


  card_info <- html %>%
    html_elements(".card-content") %>%
    html_element(".card-info")

  card_info <-
    map(card_info, function(x) {
      date <- html_elements(x, "li")[1]
      study <- html_elements(x, "li")[2]
      date <- html_text(date)
      date <- as.Date(date, "%d/%m/%Y")
      study <- html_text(study)
      study <- gsub("^Estudio\\s+", "", study)
      tibble(study, date)
    }) %>%
    list_rbind()


  out <- tibble(
    title = titles,
    url = urls
  )
  out <- cbind(card_info, out)
  out <- as_tibble(out)
}

#' Parse CIS data series search results
#'
#' @param resp The HTTP response object from the CIS search.
#'
#' @return A tibble with the parsed data series information.
#'
#' @keywords internal
parse_serie <- function(resp) {
  html <- content(resp, "parsed")
  a <-
    html %>%
    html_elements(".card-content") %>%
    html_element("a")

  titles <- html_attr(a, name = "title")
  urls <- html_attr(a, "href")

  card_info <- html %>%
    html_elements(".card-content") %>%
    html_element(".card-info")

  card_info <-
    map(card_info, function(x) {
      from <- html_elements(x, "li")[1]
      to <- html_elements(x, "li")[2]
      serie <- html_elements(x, "li")[3]

      from <- html_text(from)
      from <- as.Date(from, "%d/%m/%Y")
      to <- html_text(to)
      to <- as.Date(to, "%d/%m/%Y")
      serie <- html_text(serie)
      serie <- gsub("^Serie\\s+", "", serie)
      tibble(serie, from, to)
    }) %>%
    list_rbind()


  out <- tibble(
    title = titles,
    url = urls
  )
  out <- cbind(card_info, out)
  out <- as_tibble(out)
}


#' Parse CIS question search results
#'
#' @param resp The HTTP response object from the CIS search.
#'
#' @return A tibble with the parsed data series information.
#'
#' @keywords internal
parse_question <- function(resp) {
  html <- content(resp, "parsed")

  card_content <-
    html %>%
    html_elements(".card-content")

  a <- card_content %>%
    html_element("a")

  titles <- html_attr(a, name = "title")
  urls <- html_attr(a, "href")

  card_info <- html %>%
    html_elements(".card-content") %>%
    html_element(".card-info")

  card_info <-
    map(card_info, function(x) {
      date <- html_elements(x, "li")[1]
      question <- html_elements(x, "li")[2]
      date <- html_text(date)
      date <- as.Date(date, "%d/%m/%Y")
      question <- html_text(question)
      question <- gsub("^Pregunta\\s+", "", question)
      tibble(question, date)
    }) %>%
    list_rbind()

  out <- tibble(
    title = titles,
    url = urls
  )
  out <- cbind(card_info, out)
  out <- as_tibble(out)
}


#' Get the URL of a CIS study
#'
#' Retrieves the URL of a specific CIS study using its study ID.
#'
#' @param study_code A string with the study ID.
#'
#' @return A string with the URL of the study, or NULL if not found.
#'
#' @keywords internal
get_study_url <- function(study_code) {
  studies <- search_cis(q = study_code)

  if (is.null(studies)) {
    message("Could not retrieve study index.")
    return(NULL)
  }

  if (nrow(studies) == 0) {
    message("No study found.")
    return(NULL)
  }


  match_study <- studies[studies$study == study_code, ]

  if (nrow(match_study) > 1) {
    message("More than one study found. Returning the first one.")
    return(match_study$url[1])
  }

  if (nrow(match_study) == 0) {
    message("No study found.")
    return(NULL)
  }

  return(match_study$url)
}


#' Build CIS catalog URL with date range
#'
#' Constructs a URL for querying the CIS catalog with optional date range filters.
#'
#' @param start Integer. The starting page for the search results. Default is 1, iterate to get more results.
#' @param q String. The search query. Default is an empty string.
#' @param from Date or NULL. The start date for filtering results. Default is NULL
#' @param to Date or NULL. The end date for filtering results. Default is NULL.
#' @param sort String. The sorting order for the results ("publishDate-", "publishDate+", "relevance").
#' Default is "relevance".
#' @param catalogo String. The catalog type ("estudio", "pregunta", "serie"). Default is "estudio".
#' @param ... Additional parameters (not used).
#'
#' @return A string representing the constructed URL.
#'
#' @keywords internal
cis_catalog_url_date <- function(
  start = 1,
  q = "",
  from = NULL, # Date o NULL
  to = NULL, # Date o NULL
  sort = "relevance",
  catalogo = "estudio",
  ...
) {
  if (sort == "relevance") {
    sort <- ""
  }

  base <- "https://www.cis.es/es/estudios/catalogo"

  # Construir from / fromDate
  if (!is.null(from)) {
    from <- as.Date(from)
    stopifnot(inherits(from, "Date"))
    from_str <- format(from, "%Y-%m-%d")
    from_range <- sprintf("[%s000000 now]", format(from, "%Y%m%d"))
    fromDate_val <- from_str
  } else {
    from_range <- ""
    fromDate_val <- ""
  }

  # Construir to / toDate
  if (!is.null(to)) {
    to <- as.Date(to)
    stopifnot(inherits(to, "Date"))
    to_str <- format(to, "%Y-%m-%d")
    # Aquí puedes ajustar la lógica según cómo quieras el rango.
    # Ejemplo: desde hace 100 años hasta 'to' a las 23:59:59
    to_range <- sprintf("[now-100y %s235959]", format(to, "%Y%m%d"))
    toDate_val <- to_str
  } else {
    # Sin límite explícito: mismo comportamiento raro/flexible que usan ellos
    to_range <- "[now-100y ]"
    toDate_val <- ""
  }

  params <- list(
    start     = start,
    q         = q,
    fromDate  = fromDate_val,
    from      = from_range,
    toDate    = toDate_val,
    to        = to_range,
    sort      = sort,
    catalogo  = catalogo
  )

  modify_url(base, query = params)
}


#' Search all CIS results with automatic pagination
#'
#' Calls \code{\link{search_cis}} repeatedly, incrementing the page index until
#' no more results are returned, and returns all results in a single tibble.
#'
#' @param q String. The search query. Default is an empty string.
#' @param from Date or NULL. The start date for filtering results. Default is NULL.
#'   The date format must be "YYYY-MM-DD".
#' @param to Date or NULL. The end date for filtering results. Default is NULL.
#'   The date format must be "YYYY-MM-DD".
#' @param sort String. The sorting order for the results
#'   (\code{"publishDate-"}, \code{"publishDate+"}, \code{"relevance"}).
#'   Default is \code{"relevance"}.
#' @param catalogo String. The catalog type (\code{"estudio"}, \code{"pregunta"},
#'   \code{"serie"}). Default is \code{"estudio"}.
#' @param ... Additional parameters passed to \code{\link{search_cis}}.
#'
#' @return A tibble with all search results across all pages.
#'
#' @export
#'
#' @example R/examples/search_all_cis.R
search_all_cis <- function(
  q = "",
  from = NULL,
  to = NULL,
  sort = "relevance",
  catalogo = "estudio",
  ...
) {
  all_results <- list()
  page <- 1

  repeat {
    results <- search_cis(
      start    = page,
      q        = q,
      from     = from,
      to       = to,
      sort     = sort,
      catalogo = catalogo,
      ...
    )

    if (is.null(results) || nrow(results) == 0) break

    all_results[[page]] <- results
    page <- page + 1
  }

  if (length(all_results) == 0) {
    return(tibble())
  }

  list_rbind(all_results)
}


#' Get metadata of a CIS study
#'
#' Retrieves the technical metadata of a CIS study from its detail page,
#' including study dates, type, country, author, and thematic indices.
#'
#' @param study_code A string with the study code.
#'
#' @return A tibble with two columns: \code{field} and \code{value}.
#'
#' @export
#'
#' @example R/examples/get_metadata.R
get_metadata <- function(study_code) {
  url <- get_study_url(study_code)
  if (is.null(url)) {
    stop("Study '", study_code, "' not found.")
  }

  resp <- cis_get(url, required = TRUE, context = "retrieving study metadata")
  html <- content(resp, "parsed")

  # --- Date fields (.jdt-study-dates__item) ---
  date_labels <- trimws(html_text(html_elements(html, ".jdt-study-dates__label")))
  date_values <- trimws(html_text(html_elements(html, ".jdt-study-dates__date")))

  # --- Info items: each .last-container has a bold label + plain value ---
  info_items <- html_elements(html, ".jdt-detalle .last-container")
  info_labels <- character(0)
  info_values <- character(0)

  for (item in info_items) {
    ps <- html_elements(item, "p")
    if (length(ps) < 2) next
    label <- trimws(html_text(ps[[1]]))
    # For items with a list (e.g. "Indices tematicos"), prefer the list text
    lis <- html_elements(item, "li")
    if (length(lis) > 0) {
      value <- paste(trimws(html_text(lis)), collapse = "; ")
    } else {
      value <- trimws(html_text(ps[[2]]))
    }
    info_labels <- c(info_labels, label)
    info_values <- c(info_values, value)
  }

  tibble(
    field = c(date_labels, info_labels),
    value = c(date_values, info_values)
  )
}

test_that("cis_catalog_url_date builds URL with expected query params", {
    url <- opencis:::cis_catalog_url_date(
        start = 2,
        q = "postelectoral",
        from = "2020-01-01",
        to = "2020-12-31",
        sort = "publishDate-",
        catalogo = "estudio"
    )

    expect_match(url, "start=2")
    expect_match(url, "q=postelectoral")
    expect_match(url, "fromDate=2020-01-01")
    expect_match(url, "toDate=2020-12-31")
    expect_match(url, "catalogo=estudio")
})

\donttest{
# Retrieve all postelectoral studies (all pages)
all_studies <- search_all_cis(q = "postelectoral")
print(nrow(all_studies))

# Filter by date range
studies_2010_2020 <- search_all_cis(
  q    = "ideologia",
  from = "2010-01-01",
  to   = "2020-12-31"
)
print(studies_2010_2020)
}

---
title: "How to use opencis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to use opencis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

```

## Installation

You can install the released version of `opencis` from CRAN:

```{r, eval=FALSE}
install.packages("opencis")
```

You can install the development version from GitHub using the `remotes` package:

```{r, eval=FALSE}
remotes::install_github("hmeleiro/opencis")
```

## Usage

### Searching for studies, questions and series

`search_cis()` searches the CIS catalogue and returns a tibble with matching results. The `catalogo` argument controls what type of item is searched: `"estudio"` (default), `"pregunta"` or `"serie"`. You can restrict results to a date range with `from` and `to`, and change the sort order with `sort` (`"relevance"`, `"publishDate-"`, `"publishDate+"`).

```{r, eval = FALSE}
library(opencis)

# Search for survey studies
search_cis(q = "preelectoral", from = "2020-01-01", to = "2023-11-17")

# Search for survey questions
search_cis(q = "feminismo", catalogo = "pregunta")

# Search for data series
search_cis(q = "situación económica", catalogo = "serie")
```

By default `search_cis()` returns only the first page of results. Use `search_all_cis()` to automatically paginate through all pages and get every matching result in a single tibble:

```{r, eval = FALSE}
# Retrieve all postelectoral studies (all pages)
all_studies <- search_all_cis(q = "postelectoral")
print(nrow(all_studies))

# Filter by date range across all pages
studies <- search_all_cis(
  q    = "ideologia",
  from = "2010-01-01",
  to   = "2020-12-31"
)
```

`search_all_cis()` accepts the same arguments as `search_cis()`.

------------------------------------------------------------------------

### Reading study data into R

`read_cis()` downloads the SPSS data file for a study and imports it directly into R as a labelled data frame (via `haven`):

```{r, eval = FALSE}
df <- read_cis(3411)
print(df)
```

------------------------------------------------------------------------

### Exploring variables: the data dictionary

After loading a study with `read_cis()`, use `get_data_dictionary()` to obtain a tidy tibble with every variable name, its label and its value labels:

```{r, eval = FALSE}
df   <- read_cis(3328)
dict <- get_data_dictionary(df)
print(dict)

# Find variables whose label contains a keyword
dict[grepl("sexo", dict$label, ignore.case = TRUE), ]

# Inspect value labels for a specific variable
dict$value_labels[[which(dict$variable == "SEXO")]]
```

------------------------------------------------------------------------

### Getting study metadata

`get_metadata()` retrieves the technical information sheet of a study from the CIS website — field dates, study type, country, authorship, thematic indices, etc. — and returns it as a two-column tibble (`field`, `value`):

```{r, eval = FALSE}
meta <- get_metadata(3328)
print(meta)

```

------------------------------------------------------------------------

### Downloading the ZIP file to disk

If you want to keep the raw data files instead of reading them into a temporary directory, use `download_study()`. It saves the ZIP archive to any local folder:

```{r, eval = FALSE}
# Save to the current working directory
path <- download_study(3328)
cat("Saved to:", path, "\n")

# Save to a specific folder
path <- download_study(3328, destdir = "data/raw")
cat("Saved to:", path, "\n")
```

------------------------------------------------------------------------

### Browsing the questionnaire and technical sheet

`browse_pdf()` extracts the PDF documents bundled inside the study ZIP and opens them in your default browser. CIS ZIPs typically include two PDFs:

- **Questionnaire** (`wanted_file = "cues"`, default)
- **Technical sheet** (`wanted_file = "ft"`)

```{r, eval = FALSE}
# Open the questionnaire for study 3328
browse_pdf(3328)

# Open the technical sheet
browse_pdf(3328, wanted_file = "ft")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{get_metadata}
\alias{get_metadata}
\title{Get metadata of a CIS study}
\usage{
get_metadata(study_code)
}
\arguments{
\item{study_code}{A string with the study code.}
}
\value{
A tibble with two columns: \code{field} and \code{value}.
}
\description{
Retrieves the technical metadata of a CIS study from its detail page,
including study dates, type, country, author, and thematic indices.
}
\examples{
\donttest{
# Get metadata for study 3328
meta <- get_metadata("3328")
print(meta)

# Access a specific field
meta$value[meta$field == "Tipo de estudio"]
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{parse_serie}
\alias{parse_serie}
\title{Parse CIS data series search results}
\usage{
parse_serie(resp)
}
\arguments{
\item{resp}{The HTTP response object from the CIS search.}
}
\value{
A tibble with the parsed data series information.
}
\description{
Parse CIS data series search results
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{cis_catalog_url_date}
\alias{cis_catalog_url_date}
\title{Build CIS catalog URL with date range}
\usage{
cis_catalog_url_date(
  start = 1,
  q = "",
  from = NULL,
  to = NULL,
  sort = "relevance",
  catalogo = "estudio",
  ...
)
}
\arguments{
\item{start}{Integer. The starting page for the search results. Default is 1, iterate to get more results.}

\item{q}{String. The search query. Default is an empty string.}

\item{from}{Date or NULL. The start date for filtering results. Default is NULL}

\item{to}{Date or NULL. The end date for filtering results. Default is NULL.}

\item{sort}{String. The sorting order for the results ("publishDate-", "publishDate+", "relevance").
Default is "relevance".}

\item{catalogo}{String. The catalog type ("estudio", "pregunta", "serie"). Default is "estudio".}

\item{...}{Additional parameters (not used).}
}
\value{
A string representing the constructed URL.
}
\description{
Constructs a URL for querying the CIS catalog with optional date range filters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{browse_pdf}
\alias{browse_pdf}
\title{Open the questionnaire PDF of a CIS study}
\usage{
browse_pdf(study_code, wanted_file = "cues")
}
\arguments{
\item{study_code}{A string with the study code.}

\item{wanted_file}{A keyword used to match the PDF filename inside the ZIP.
Use \code{"cues"} (default) for the questionnaire or \code{"ft"} for the
technical sheet.}
}
\value{
Called for its side effect of opening the PDF in the browser.
Returns \code{NULL} invisibly.
}
\description{
Opens a PDF document from a CIS study in the default browser.
}
\details{
CIS study ZIP files typically contain two PDF documents:
\itemize{
\item The \strong{questionnaire} (cuestionario): use \code{wanted_file = "cues"}.
\item The \strong{technical sheet} (ficha técnica): use \code{wanted_file = "ft"}.
}
}
\examples{
if (interactive()) {
# Open the questionnaire (cuestionario) for study 3328
browse_pdf("3328")

# Open the technical sheet (ficha técnica) for study 3328
browse_pdf("3328", wanted_file = "ft")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{read_cis}
\alias{read_cis}
\title{Import a CIS study}
\usage{
read_cis(study_code)
}
\arguments{
\item{study_code}{A string with the study code.}
}
\value{
A data.frame with the study data.
}
\description{
Download and import the data of a CIS study.
}
\examples{
\donttest{
# If you know the study code you can just read it into R
df <- read_cis("3328")
print(df)

# If you dont know the study code, you can search for a study using search_cis() function:
studies <- search_cis(q = "gastronomia")
print(studies)

df <- read_cis(studies$study[1])
print(df)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{get_study_url}
\alias{get_study_url}
\title{Get the URL of a CIS study}
\usage{
get_study_url(study_code)
}
\arguments{
\item{study_code}{A string with the study ID.}
}
\value{
A string with the URL of the study, or NULL if not found.
}
\description{
Retrieves the URL of a specific CIS study using its study ID.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{clear_cache}
\alias{clear_cache}
\title{Clear the opencis session cache}
\usage{
clear_cache()
}
\value{
\code{NULL} invisibly.
}
\description{
Clears the in-memory cache used by \code{\link{search_cis}} and
\code{\link{read_cis}}. Call this when you want to force fresh data
to be retrieved from the CIS server within the same R session.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opencis-package.R
\docType{package}
\name{opencis-package}
\alias{opencis}
\alias{opencis-package}
\title{opencis: Import Data from Spanish Sociological Research Center (CIS)}
\description{
Search and import data directly to R from the Spanish Sociological Research Center (CIS) \url{https://www.cis.es/inicio}. The CIS is a public institution that conducts electoral and sociological research studies on the Spanish society. The CIS has a large database of surveys that can be accessed through its website. The package includes functions to search for surveys, survey questions and timeseries, and import the data directly to R.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://opencis.spainelectoralproject.com}
  \item \url{https://github.com/hmeleiro/opencis}
  \item Report bugs at \url{https://github.com/hmeleiro/opencis/issues}
}

}
\author{
\strong{Maintainer}: Héctor Meleiro \email{hmeleiros@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{parse_study}
\alias{parse_study}
\title{Parse CIS study search results}
\usage{
parse_study(resp)
}
\arguments{
\item{resp}{The HTTP response object from the CIS search.}
}
\value{
A tibble with the parsed studies information.
}
\description{
Parse CIS study search results
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{download_study}
\alias{download_study}
\title{Download a CIS study ZIP file to disk}
\usage{
download_study(study_code, destdir = ".")
}
\arguments{
\item{study_code}{A string with the study code.}

\item{destdir}{A string with the directory where the ZIP file will be saved.
Defaults to the current working directory.}
}
\value{
The path to the saved ZIP file, invisibly.
}
\description{
Downloads the data ZIP file for a CIS study to a specified directory,
instead of a temporary folder. Useful for projects that need to keep
the raw data files.
}
\examples{
\donttest{
# Save the ZIP file to a temporary directory
path <- download_study("3328", destdir = tempdir())
cat("Saved to:", path, "\n")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{find_url}
\alias{find_url}
\title{Find CIS study data URL in HTML content}
\usage{
find_url(html, ids = NULL, allow_uuid = FALSE)
}
\arguments{
\item{html}{A character string containing the HTML content to search.}

\item{ids}{An optional vector of two strings representing the numeric subroutes of the URL
(e.g., c("3411", "3411") for "https://www.cis.es/documents/3411/3411/MD3411.zip").
If NULL, the function searches for any valid CIS study data URL.}

\item{allow_uuid}{A boolean indicating whether to allow UUIDs in the URL.
Defaults to FALSE.}
}
\value{
A character vector of unique URLs found in the HTML content.
}
\description{
Searches for the URL of the CIS study data ZIP file within the provided HTML content.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{get_data_dictionary}
\alias{get_data_dictionary}
\title{Extract a data dictionary from a CIS study data frame}
\usage{
get_data_dictionary(data)
}
\arguments{
\item{data}{A data.frame loaded from a CIS \code{.sav} file,
typically the output of \code{\link{read_cis}}.}
}
\value{
A tibble with columns:
\describe{
\item{variable}{Variable name.}
\item{label}{Variable label, or \code{NA} if none.}
\item{value_labels}{A named numeric vector of value labels,
or \code{NULL} for unlabelled variables (list-column).}
}
}
\description{
Returns a tibble listing each variable in the data along with its
variable label and value labels, as loaded by \code{haven}.
}
\examples{
# Create a small labelled data frame
df <- data.frame(
  SEXO = haven::labelled(c(1, 2, 1), labels = c(Hombre = 1, Mujer = 2)),
  EDAD = c(34, 51, 29)
)
attr(df$SEXO, "label") <- "Sexo"
attr(df$EDAD, "label") <- "Edad"

# Inspect its variable dictionary
dict <- get_data_dictionary(df)
print(dict)

# Find variables with a specific keyword in their label
dict[grepl("sexo", dict$label, ignore.case = TRUE), ]

# Inspect value labels for a specific variable
sex_var <- match("SEXO", dict$variable)
if (!is.na(sex_var)) {
  dict$value_labels[[sex_var]]
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{read_sav_from_zip}
\alias{read_sav_from_zip}
\title{Read a SAV file from a ZIP archive}
\usage{
read_sav_from_zip(zip_path)
}
\arguments{
\item{zip_path}{A string with the path to the ZIP file.}
}
\value{
A data.frame with the data read from the .sav file.
}
\description{
Extracts and reads the SPSS (.sav) data file contained in a ZIP archive.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{download_file}
\alias{download_file}
\title{Download and read a CIS study from a given URL}
\usage{
download_file(url, destfile = tempfile(fileext = ".zip"))
}
\arguments{
\item{url}{A string with the URL of the CIS study page.}

\item{destfile}{A string with the path where the ZIP file will be saved.
Defaults to a temporary file.}
}
\description{
Download and read a CIS study from a given URL
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imports.R
\name{opencis-imports}
\alias{opencis-imports}
\title{Imports}
\description{
Package import declarations
}
\details{
Package imports

These declarations ensure NAMESPACE keeps required imports.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{search_cis}
\alias{search_cis}
\title{Search for CIS studies.}
\usage{
search_cis(
  start = 1,
  q = "",
  from = NULL,
  to = NULL,
  sort = "relevance",
  catalogo = "estudio",
  ...
)
}
\arguments{
\item{start}{Integer. The starting page for the search results. Default is 1, iterate to get more results.}

\item{q}{String. The search query. Default is an empty string.}

\item{from}{Date or NULL. The start date for filtering results. Default is NULL. The date format must be "YYYY-MM-DD".}

\item{to}{Date or NULL. The end date for filtering results. Default is NULL. The date format must be "YYYY-MM-DD".}

\item{sort}{String. The sorting order for the results ("publishDate-", "publishDate+", "relevance").
Default is "relevance".}

\item{catalogo}{String. The catalog type ("estudio", "pregunta", "serie"). Default is "estudio".}

\item{...}{Additional parameters (not used).}
}
\value{
A data.frame with the search results.
}
\description{
Searches for CIS studies using the CIS search engine.
}
\examples{
\donttest{
# Search by search terms
studies <- search_cis(q = "postelectoral")
print(studies)

# Narrow the search by dates
studies <- search_cis(q = "postelectoral",
                          from = "2011-01-01",
                          to = "2020-01-01")
print(studies)

# Use the catalogo parameter to search for questions ("pregunta") or data series ("serie")
studies <- search_cis(q = "ideologia",
                          from = "2011-01-01",
                          to = "2020-01-01",
                          catalogo = "serie")
print(studies)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{search_all_cis}
\alias{search_all_cis}
\title{Search all CIS results with automatic pagination}
\usage{
search_all_cis(
  q = "",
  from = NULL,
  to = NULL,
  sort = "relevance",
  catalogo = "estudio",
  ...
)
}
\arguments{
\item{q}{String. The search query. Default is an empty string.}

\item{from}{Date or NULL. The start date for filtering results. Default is NULL.
The date format must be "YYYY-MM-DD".}

\item{to}{Date or NULL. The end date for filtering results. Default is NULL.
The date format must be "YYYY-MM-DD".}

\item{sort}{String. The sorting order for the results
(\code{"publishDate-"}, \code{"publishDate+"}, \code{"relevance"}).
Default is \code{"relevance"}.}

\item{catalogo}{String. The catalog type (\code{"estudio"}, \code{"pregunta"},
\code{"serie"}). Default is \code{"estudio"}.}

\item{...}{Additional parameters passed to \code{\link{search_cis}}.}
}
\value{
A tibble with all search results across all pages.
}
\description{
Calls \code{\link{search_cis}} repeatedly, incrementing the page index until
no more results are returned, and returns all results in a single tibble.
}
\examples{
\donttest{
# Retrieve all postelectoral studies (all pages)
all_studies <- search_all_cis(q = "postelectoral")
print(nrow(all_studies))

# Filter by date range
studies_2010_2020 <- search_all_cis(
  q    = "ideologia",
  from = "2010-01-01",
  to   = "2020-12-31"
)
print(studies_2010_2020)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{list_file_paths}
\alias{list_file_paths}
\title{List file paths inside a ZIP archive}
\usage{
list_file_paths(zip_file, type = NULL)
}
\arguments{
\item{zip_file}{A string with the path to the ZIP file.}

\item{type}{A string with the file extension to filter by (e.g. \code{"sav"}, \code{"pdf"}).
If \code{NULL} (default), all files are returned.}
}
\value{
A data.frame with the files in the ZIP archive.
}
\description{
Returns a data.frame with the files contained in a ZIP archive,
optionally filtered by file extension.
}
\keyword{internal}
